# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.created_at import CreatedAt
from ..types.customer_number_nullable import CustomerNumberNullable
from ..types.customer_query_records import CustomerQueryRecords
from ..types.customer_status import CustomerStatus
from ..types.email import Email
from ..types.entrypointfield import Entrypointfield
from ..types.idempotency_key import IdempotencyKey
from ..types.identifierfields import Identifierfields
from ..types.payabli_api_response_00_responsedatanonobject import PayabliApiResponse00Responsedatanonobject
from ..types.payabli_api_response_customer_query import PayabliApiResponseCustomerQuery
from ..types.shippingaddress import Shippingaddress
from ..types.shippingaddressadditional import Shippingaddressadditional
from ..types.shippingcity import Shippingcity
from ..types.shippingcountry import Shippingcountry
from ..types.shippingstate import Shippingstate
from ..types.shippingzip import Shippingzip
from ..types.timezone import Timezone
from .raw_client import AsyncRawCustomerClient, RawCustomerClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomerClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCustomerClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCustomerClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCustomerClient
        """
        return self._raw_client

    def add_customer(
        self,
        entry: Entrypointfield,
        *,
        force_customer_creation: typing.Optional[bool] = None,
        replace_existing: typing.Optional[int] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        customer_number: typing.Optional[CustomerNumberNullable] = OMIT,
        customer_username: typing.Optional[str] = OMIT,
        customer_psw: typing.Optional[str] = OMIT,
        customer_status: typing.Optional[CustomerStatus] = OMIT,
        company: typing.Optional[str] = OMIT,
        firstname: typing.Optional[str] = OMIT,
        lastname: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        email: typing.Optional[Email] = OMIT,
        address: typing.Optional[str] = OMIT,
        address_1: typing.Optional[str] = OMIT,
        city: typing.Optional[str] = OMIT,
        state: typing.Optional[str] = OMIT,
        zip: typing.Optional[str] = OMIT,
        country: typing.Optional[str] = OMIT,
        shipping_address: typing.Optional[Shippingaddress] = OMIT,
        shipping_address_1: typing.Optional[Shippingaddressadditional] = OMIT,
        shipping_city: typing.Optional[Shippingcity] = OMIT,
        shipping_state: typing.Optional[Shippingstate] = OMIT,
        shipping_zip: typing.Optional[Shippingzip] = OMIT,
        shipping_country: typing.Optional[Shippingcountry] = OMIT,
        balance: typing.Optional[float] = OMIT,
        time_zone: typing.Optional[Timezone] = OMIT,
        additional_fields: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        identifier_fields: typing.Optional[Identifierfields] = OMIT,
        created_at: typing.Optional[CreatedAt] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseCustomerQuery:
        """
        Creates a customer in an entrypoint. An identifier is required to create customer records. Change your identifier settings in Settings > Custom Fields in PartnerHub.
        If you don't include an identifier, the record is rejected.

        Parameters
        ----------
        entry : Entrypointfield

        force_customer_creation : typing.Optional[bool]
            When `true`, the request creates a new customer record, regardless of whether customer identifiers match an existing customer.

        replace_existing : typing.Optional[int]
            Flag indicating to replace existing customer with a new record. Possible values: 0 (don't replace), 1 (replace). Default is `0`.

        idempotency_key : typing.Optional[IdempotencyKey]

        customer_number : typing.Optional[CustomerNumberNullable]

        customer_username : typing.Optional[str]
            Customer username for customer portal

        customer_psw : typing.Optional[str]
            Customer password for customer portal

        customer_status : typing.Optional[CustomerStatus]

        company : typing.Optional[str]
            Company name

        firstname : typing.Optional[str]
            Customer first name

        lastname : typing.Optional[str]
            Customer last name

        phone : typing.Optional[str]
            Customer phone number

        email : typing.Optional[Email]
            Customer email address.

        address : typing.Optional[str]
            Customer address

        address_1 : typing.Optional[str]
            Additional customer address

        city : typing.Optional[str]
            Customer city

        state : typing.Optional[str]
            Customer State

        zip : typing.Optional[str]
            Customer postal code

        country : typing.Optional[str]
            Customer country in ISO-3166-1 alpha 2 format. See https://en.wikipedia.org/wiki/ISO_3166-1 for reference.

        shipping_address : typing.Optional[Shippingaddress]

        shipping_address_1 : typing.Optional[Shippingaddressadditional]

        shipping_city : typing.Optional[Shippingcity]

        shipping_state : typing.Optional[Shippingstate]

        shipping_zip : typing.Optional[Shippingzip]

        shipping_country : typing.Optional[Shippingcountry]

        balance : typing.Optional[float]
            Customer balance.

        time_zone : typing.Optional[Timezone]

        additional_fields : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Additional Custom fields in format "key":"value".

        identifier_fields : typing.Optional[Identifierfields]

        created_at : typing.Optional[CreatedAt]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseCustomerQuery
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.customer.add_customer(
            entry="8cfec329267",
            customer_number="12356ACB",
            firstname="Irene",
            lastname="Canizales",
            address_1="123 Bishop's Trail",
            city="Mountain City",
            state="TN",
            zip="37612",
            country="US",
            email="irene@canizalesconcrete.com",
            identifier_fields=["email"],
            time_zone=-5,
        )
        """
        _response = self._raw_client.add_customer(
            entry,
            force_customer_creation=force_customer_creation,
            replace_existing=replace_existing,
            idempotency_key=idempotency_key,
            customer_number=customer_number,
            customer_username=customer_username,
            customer_psw=customer_psw,
            customer_status=customer_status,
            company=company,
            firstname=firstname,
            lastname=lastname,
            phone=phone,
            email=email,
            address=address,
            address_1=address_1,
            city=city,
            state=state,
            zip=zip,
            country=country,
            shipping_address=shipping_address,
            shipping_address_1=shipping_address_1,
            shipping_city=shipping_city,
            shipping_state=shipping_state,
            shipping_zip=shipping_zip,
            shipping_country=shipping_country,
            balance=balance,
            time_zone=time_zone,
            additional_fields=additional_fields,
            identifier_fields=identifier_fields,
            created_at=created_at,
            request_options=request_options,
        )
        return _response.data

    def delete_customer(
        self, customer_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Delete a customer record.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.customer.delete_customer(
            customer_id=998,
        )
        """
        _response = self._raw_client.delete_customer(customer_id, request_options=request_options)
        return _response.data

    def get_customer(
        self, customer_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CustomerQueryRecords:
        """
        Retrieves a customer's record and details.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CustomerQueryRecords
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.customer.get_customer(
            customer_id=998,
        )
        """
        _response = self._raw_client.get_customer(customer_id, request_options=request_options)
        return _response.data

    def link_customer_transaction(
        self, customer_id: int, trans_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Links a customer to a transaction by ID.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        trans_id : str
            ReferenceId for the transaction (PaymentId).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.customer.link_customer_transaction(
            customer_id=998,
            trans_id="45-as456777hhhhhhhhhh77777777-324",
        )
        """
        _response = self._raw_client.link_customer_transaction(customer_id, trans_id, request_options=request_options)
        return _response.data

    def request_consent(
        self, customer_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Sends the consent opt-in email to the customer email address in the customer record.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.customer.request_consent(
            customer_id=998,
        )
        """
        _response = self._raw_client.request_consent(customer_id, request_options=request_options)
        return _response.data

    def update_customer(
        self,
        customer_id: int,
        *,
        customer_number: typing.Optional[CustomerNumberNullable] = OMIT,
        customer_username: typing.Optional[str] = OMIT,
        customer_psw: typing.Optional[str] = OMIT,
        customer_status: typing.Optional[CustomerStatus] = OMIT,
        company: typing.Optional[str] = OMIT,
        firstname: typing.Optional[str] = OMIT,
        lastname: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        email: typing.Optional[Email] = OMIT,
        address: typing.Optional[str] = OMIT,
        address_1: typing.Optional[str] = OMIT,
        city: typing.Optional[str] = OMIT,
        state: typing.Optional[str] = OMIT,
        zip: typing.Optional[str] = OMIT,
        country: typing.Optional[str] = OMIT,
        shipping_address: typing.Optional[Shippingaddress] = OMIT,
        shipping_address_1: typing.Optional[Shippingaddressadditional] = OMIT,
        shipping_city: typing.Optional[Shippingcity] = OMIT,
        shipping_state: typing.Optional[Shippingstate] = OMIT,
        shipping_zip: typing.Optional[Shippingzip] = OMIT,
        shipping_country: typing.Optional[Shippingcountry] = OMIT,
        balance: typing.Optional[float] = OMIT,
        time_zone: typing.Optional[Timezone] = OMIT,
        additional_fields: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        identifier_fields: typing.Optional[Identifierfields] = OMIT,
        created_at: typing.Optional[CreatedAt] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Update a customer record. Include only the fields you want to change.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        customer_number : typing.Optional[CustomerNumberNullable]

        customer_username : typing.Optional[str]
            Customer username for customer portal

        customer_psw : typing.Optional[str]
            Customer password for customer portal

        customer_status : typing.Optional[CustomerStatus]

        company : typing.Optional[str]
            Company name

        firstname : typing.Optional[str]
            Customer first name

        lastname : typing.Optional[str]
            Customer last name

        phone : typing.Optional[str]
            Customer phone number

        email : typing.Optional[Email]
            Customer email address.

        address : typing.Optional[str]
            Customer address

        address_1 : typing.Optional[str]
            Additional customer address

        city : typing.Optional[str]
            Customer city

        state : typing.Optional[str]
            Customer State

        zip : typing.Optional[str]
            Customer postal code

        country : typing.Optional[str]
            Customer country in ISO-3166-1 alpha 2 format. See https://en.wikipedia.org/wiki/ISO_3166-1 for reference.

        shipping_address : typing.Optional[Shippingaddress]

        shipping_address_1 : typing.Optional[Shippingaddressadditional]

        shipping_city : typing.Optional[Shippingcity]

        shipping_state : typing.Optional[Shippingstate]

        shipping_zip : typing.Optional[Shippingzip]

        shipping_country : typing.Optional[Shippingcountry]

        balance : typing.Optional[float]
            Customer balance.

        time_zone : typing.Optional[Timezone]

        additional_fields : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Additional Custom fields in format "key":"value".

        identifier_fields : typing.Optional[Identifierfields]

        created_at : typing.Optional[CreatedAt]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.customer.update_customer(
            customer_id=998,
            firstname="Irene",
            lastname="Canizales",
            address_1="145 Bishop's Trail",
            city="Mountain City",
            state="TN",
            zip="37612",
            country="US",
        )
        """
        _response = self._raw_client.update_customer(
            customer_id,
            customer_number=customer_number,
            customer_username=customer_username,
            customer_psw=customer_psw,
            customer_status=customer_status,
            company=company,
            firstname=firstname,
            lastname=lastname,
            phone=phone,
            email=email,
            address=address,
            address_1=address_1,
            city=city,
            state=state,
            zip=zip,
            country=country,
            shipping_address=shipping_address,
            shipping_address_1=shipping_address_1,
            shipping_city=shipping_city,
            shipping_state=shipping_state,
            shipping_zip=shipping_zip,
            shipping_country=shipping_country,
            balance=balance,
            time_zone=time_zone,
            additional_fields=additional_fields,
            identifier_fields=identifier_fields,
            created_at=created_at,
            request_options=request_options,
        )
        return _response.data


class AsyncCustomerClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCustomerClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCustomerClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCustomerClient
        """
        return self._raw_client

    async def add_customer(
        self,
        entry: Entrypointfield,
        *,
        force_customer_creation: typing.Optional[bool] = None,
        replace_existing: typing.Optional[int] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        customer_number: typing.Optional[CustomerNumberNullable] = OMIT,
        customer_username: typing.Optional[str] = OMIT,
        customer_psw: typing.Optional[str] = OMIT,
        customer_status: typing.Optional[CustomerStatus] = OMIT,
        company: typing.Optional[str] = OMIT,
        firstname: typing.Optional[str] = OMIT,
        lastname: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        email: typing.Optional[Email] = OMIT,
        address: typing.Optional[str] = OMIT,
        address_1: typing.Optional[str] = OMIT,
        city: typing.Optional[str] = OMIT,
        state: typing.Optional[str] = OMIT,
        zip: typing.Optional[str] = OMIT,
        country: typing.Optional[str] = OMIT,
        shipping_address: typing.Optional[Shippingaddress] = OMIT,
        shipping_address_1: typing.Optional[Shippingaddressadditional] = OMIT,
        shipping_city: typing.Optional[Shippingcity] = OMIT,
        shipping_state: typing.Optional[Shippingstate] = OMIT,
        shipping_zip: typing.Optional[Shippingzip] = OMIT,
        shipping_country: typing.Optional[Shippingcountry] = OMIT,
        balance: typing.Optional[float] = OMIT,
        time_zone: typing.Optional[Timezone] = OMIT,
        additional_fields: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        identifier_fields: typing.Optional[Identifierfields] = OMIT,
        created_at: typing.Optional[CreatedAt] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseCustomerQuery:
        """
        Creates a customer in an entrypoint. An identifier is required to create customer records. Change your identifier settings in Settings > Custom Fields in PartnerHub.
        If you don't include an identifier, the record is rejected.

        Parameters
        ----------
        entry : Entrypointfield

        force_customer_creation : typing.Optional[bool]
            When `true`, the request creates a new customer record, regardless of whether customer identifiers match an existing customer.

        replace_existing : typing.Optional[int]
            Flag indicating to replace existing customer with a new record. Possible values: 0 (don't replace), 1 (replace). Default is `0`.

        idempotency_key : typing.Optional[IdempotencyKey]

        customer_number : typing.Optional[CustomerNumberNullable]

        customer_username : typing.Optional[str]
            Customer username for customer portal

        customer_psw : typing.Optional[str]
            Customer password for customer portal

        customer_status : typing.Optional[CustomerStatus]

        company : typing.Optional[str]
            Company name

        firstname : typing.Optional[str]
            Customer first name

        lastname : typing.Optional[str]
            Customer last name

        phone : typing.Optional[str]
            Customer phone number

        email : typing.Optional[Email]
            Customer email address.

        address : typing.Optional[str]
            Customer address

        address_1 : typing.Optional[str]
            Additional customer address

        city : typing.Optional[str]
            Customer city

        state : typing.Optional[str]
            Customer State

        zip : typing.Optional[str]
            Customer postal code

        country : typing.Optional[str]
            Customer country in ISO-3166-1 alpha 2 format. See https://en.wikipedia.org/wiki/ISO_3166-1 for reference.

        shipping_address : typing.Optional[Shippingaddress]

        shipping_address_1 : typing.Optional[Shippingaddressadditional]

        shipping_city : typing.Optional[Shippingcity]

        shipping_state : typing.Optional[Shippingstate]

        shipping_zip : typing.Optional[Shippingzip]

        shipping_country : typing.Optional[Shippingcountry]

        balance : typing.Optional[float]
            Customer balance.

        time_zone : typing.Optional[Timezone]

        additional_fields : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Additional Custom fields in format "key":"value".

        identifier_fields : typing.Optional[Identifierfields]

        created_at : typing.Optional[CreatedAt]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseCustomerQuery
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.customer.add_customer(
                entry="8cfec329267",
                customer_number="12356ACB",
                firstname="Irene",
                lastname="Canizales",
                address_1="123 Bishop's Trail",
                city="Mountain City",
                state="TN",
                zip="37612",
                country="US",
                email="irene@canizalesconcrete.com",
                identifier_fields=["email"],
                time_zone=-5,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.add_customer(
            entry,
            force_customer_creation=force_customer_creation,
            replace_existing=replace_existing,
            idempotency_key=idempotency_key,
            customer_number=customer_number,
            customer_username=customer_username,
            customer_psw=customer_psw,
            customer_status=customer_status,
            company=company,
            firstname=firstname,
            lastname=lastname,
            phone=phone,
            email=email,
            address=address,
            address_1=address_1,
            city=city,
            state=state,
            zip=zip,
            country=country,
            shipping_address=shipping_address,
            shipping_address_1=shipping_address_1,
            shipping_city=shipping_city,
            shipping_state=shipping_state,
            shipping_zip=shipping_zip,
            shipping_country=shipping_country,
            balance=balance,
            time_zone=time_zone,
            additional_fields=additional_fields,
            identifier_fields=identifier_fields,
            created_at=created_at,
            request_options=request_options,
        )
        return _response.data

    async def delete_customer(
        self, customer_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Delete a customer record.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.customer.delete_customer(
                customer_id=998,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_customer(customer_id, request_options=request_options)
        return _response.data

    async def get_customer(
        self, customer_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CustomerQueryRecords:
        """
        Retrieves a customer's record and details.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CustomerQueryRecords
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.customer.get_customer(
                customer_id=998,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_customer(customer_id, request_options=request_options)
        return _response.data

    async def link_customer_transaction(
        self, customer_id: int, trans_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Links a customer to a transaction by ID.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        trans_id : str
            ReferenceId for the transaction (PaymentId).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.customer.link_customer_transaction(
                customer_id=998,
                trans_id="45-as456777hhhhhhhhhh77777777-324",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.link_customer_transaction(
            customer_id, trans_id, request_options=request_options
        )
        return _response.data

    async def request_consent(
        self, customer_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Sends the consent opt-in email to the customer email address in the customer record.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.customer.request_consent(
                customer_id=998,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.request_consent(customer_id, request_options=request_options)
        return _response.data

    async def update_customer(
        self,
        customer_id: int,
        *,
        customer_number: typing.Optional[CustomerNumberNullable] = OMIT,
        customer_username: typing.Optional[str] = OMIT,
        customer_psw: typing.Optional[str] = OMIT,
        customer_status: typing.Optional[CustomerStatus] = OMIT,
        company: typing.Optional[str] = OMIT,
        firstname: typing.Optional[str] = OMIT,
        lastname: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        email: typing.Optional[Email] = OMIT,
        address: typing.Optional[str] = OMIT,
        address_1: typing.Optional[str] = OMIT,
        city: typing.Optional[str] = OMIT,
        state: typing.Optional[str] = OMIT,
        zip: typing.Optional[str] = OMIT,
        country: typing.Optional[str] = OMIT,
        shipping_address: typing.Optional[Shippingaddress] = OMIT,
        shipping_address_1: typing.Optional[Shippingaddressadditional] = OMIT,
        shipping_city: typing.Optional[Shippingcity] = OMIT,
        shipping_state: typing.Optional[Shippingstate] = OMIT,
        shipping_zip: typing.Optional[Shippingzip] = OMIT,
        shipping_country: typing.Optional[Shippingcountry] = OMIT,
        balance: typing.Optional[float] = OMIT,
        time_zone: typing.Optional[Timezone] = OMIT,
        additional_fields: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        identifier_fields: typing.Optional[Identifierfields] = OMIT,
        created_at: typing.Optional[CreatedAt] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Update a customer record. Include only the fields you want to change.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        customer_number : typing.Optional[CustomerNumberNullable]

        customer_username : typing.Optional[str]
            Customer username for customer portal

        customer_psw : typing.Optional[str]
            Customer password for customer portal

        customer_status : typing.Optional[CustomerStatus]

        company : typing.Optional[str]
            Company name

        firstname : typing.Optional[str]
            Customer first name

        lastname : typing.Optional[str]
            Customer last name

        phone : typing.Optional[str]
            Customer phone number

        email : typing.Optional[Email]
            Customer email address.

        address : typing.Optional[str]
            Customer address

        address_1 : typing.Optional[str]
            Additional customer address

        city : typing.Optional[str]
            Customer city

        state : typing.Optional[str]
            Customer State

        zip : typing.Optional[str]
            Customer postal code

        country : typing.Optional[str]
            Customer country in ISO-3166-1 alpha 2 format. See https://en.wikipedia.org/wiki/ISO_3166-1 for reference.

        shipping_address : typing.Optional[Shippingaddress]

        shipping_address_1 : typing.Optional[Shippingaddressadditional]

        shipping_city : typing.Optional[Shippingcity]

        shipping_state : typing.Optional[Shippingstate]

        shipping_zip : typing.Optional[Shippingzip]

        shipping_country : typing.Optional[Shippingcountry]

        balance : typing.Optional[float]
            Customer balance.

        time_zone : typing.Optional[Timezone]

        additional_fields : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Additional Custom fields in format "key":"value".

        identifier_fields : typing.Optional[Identifierfields]

        created_at : typing.Optional[CreatedAt]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.customer.update_customer(
                customer_id=998,
                firstname="Irene",
                lastname="Canizales",
                address_1="145 Bishop's Trail",
                city="Mountain City",
                state="TN",
                zip="37612",
                country="US",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_customer(
            customer_id,
            customer_number=customer_number,
            customer_username=customer_username,
            customer_psw=customer_psw,
            customer_status=customer_status,
            company=company,
            firstname=firstname,
            lastname=lastname,
            phone=phone,
            email=email,
            address=address,
            address_1=address_1,
            city=city,
            state=state,
            zip=zip,
            country=country,
            shipping_address=shipping_address,
            shipping_address_1=shipping_address_1,
            shipping_city=shipping_city,
            shipping_state=shipping_state,
            shipping_zip=shipping_zip,
            shipping_country=shipping_country,
            balance=balance,
            time_zone=time_zone,
            additional_fields=additional_fields,
            identifier_fields=identifier_fields,
            created_at=created_at,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from ...types.payabli_api_response_generic_2_part import PayabliApiResponseGeneric2Part


class ModifyApprovalBillResponse(PayabliApiResponseGeneric2Part):
    response_data: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="responseData")] = (
        pydantic.Field(default=None)
    )
    """
    If `isSuccess` = true, this contains the bill identifier. If `isSuccess` = false, this contains the reason for the error. 
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .boarding_query_links import BoardingQueryLinks
from .created_at import CreatedAt
from .is_root import IsRoot
from .org_parent_name import OrgParentName
from .recipient_email_notification import RecipientEmailNotification
from .resumable import Resumable
from .template_code import TemplateCode
from .template_content_response import TemplateContentResponse


class TemplateQueryRecord(UniversalBaseModel):
    add_price: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="addPrice")] = None
    boarding_links: typing_extensions.Annotated[
        typing.Optional[typing.List[BoardingQueryLinks]], FieldMetadata(alias="boardingLinks")
    ] = None
    created_at: typing_extensions.Annotated[typing.Optional[CreatedAt], FieldMetadata(alias="createdAt")] = None
    id_template: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="idTemplate")] = None
    is_root: typing_extensions.Annotated[typing.Optional[IsRoot], FieldMetadata(alias="isRoot")] = None
    org_parent_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="orgParentName")
    ] = None
    recipient_email_notification: typing_extensions.Annotated[
        typing.Optional[RecipientEmailNotification], FieldMetadata(alias="recipientEmailNotification")
    ] = None
    resumable: typing.Optional[Resumable] = None
    template_code: typing_extensions.Annotated[typing.Optional[TemplateCode], FieldMetadata(alias="templateCode")] = (
        None
    )
    template_content: typing_extensions.Annotated[
        typing.Optional[TemplateContentResponse], FieldMetadata(alias="templateContent")
    ] = None
    template_description: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="templateDescription")
    ] = None
    template_title: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="templateTitle")] = None
    used_by: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="usedBy")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .odp_setup_processing_region import OdpSetupProcessingRegion


class OdpSetup(UniversalBaseModel):
    allow_ach: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="allowAch")] = pydantic.Field(
        default=None
    )
    """
    Enables or disables ACH payout functionality
    """

    allow_checks: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="allowChecks")] = (
        pydantic.Field(default=None)
    )
    """
    Enables or disables check printing payout functionality
    """

    allow_v_card: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="allowVCard")] = (
        pydantic.Field(default=None)
    )
    """
    Enables or disables vCard payout functionality
    """

    processing_region: typing.Optional[OdpSetupProcessingRegion] = pydantic.Field(default=None)
    """
    Region where payment processing occurs
    """

    processor: typing.Optional[str] = pydantic.Field(default=None)
    """
    Payment processor identifier
    """

    issuer_network_settings_id: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="issuerNetworkSettingsId")
    ] = pydantic.Field(default=None)
    """
    Reference ID for the program enabled for ODP issuance
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .additional_data import AdditionalData
from .billing_address_addtl_nullable import BillingAddressAddtlNullable
from .billing_address_nullable import BillingAddressNullable
from .billing_city_nullable import BillingCityNullable
from .billing_country_nullable import BillingCountryNullable
from .billing_state_nullable import BillingStateNullable
from .billing_zip import BillingZip
from .customer_id import CustomerId
from .customer_number_nullable import CustomerNumberNullable
from .email import Email
from .identifierfields import Identifierfields
from .phone_number import PhoneNumber
from .shippingaddress import Shippingaddress
from .shippingaddressadditional import Shippingaddressadditional
from .shippingcity import Shippingcity
from .shippingcountry import Shippingcountry
from .shippingstate import Shippingstate
from .shippingzip import Shippingzip


class PayorDataRequest(UniversalBaseModel):
    """
    Customer information. May be required, depending on the paypoint's settings. Required for subscriptions.
    """

    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalData], FieldMetadata(alias="additionalData")
    ] = None
    billing_address_1: typing_extensions.Annotated[
        typing.Optional[BillingAddressNullable], FieldMetadata(alias="billingAddress1")
    ] = None
    billing_address_2: typing_extensions.Annotated[
        typing.Optional[BillingAddressAddtlNullable], FieldMetadata(alias="billingAddress2")
    ] = None
    billing_city: typing_extensions.Annotated[
        typing.Optional[BillingCityNullable], FieldMetadata(alias="billingCity")
    ] = None
    billing_country: typing_extensions.Annotated[
        typing.Optional[BillingCountryNullable], FieldMetadata(alias="billingCountry")
    ] = None
    billing_email: typing_extensions.Annotated[typing.Optional[Email], FieldMetadata(alias="billingEmail")] = None
    billing_phone: typing_extensions.Annotated[typing.Optional[PhoneNumber], FieldMetadata(alias="billingPhone")] = None
    billing_state: typing_extensions.Annotated[
        typing.Optional[BillingStateNullable], FieldMetadata(alias="billingState")
    ] = None
    billing_zip: typing_extensions.Annotated[typing.Optional[BillingZip], FieldMetadata(alias="billingZip")] = (
        pydantic.Field(default=None)
    )
    """
    Customer's billing ZIP code. For Pay In functions, this field supports 5-digit and 9-digit ZIP codes and alphanumeric Canadian postal codes. For example: "37615-1234" or "37615".
    """

    company: typing.Optional[str] = pydantic.Field(default=None)
    """
    Customer's company name.
    """

    customer_id: typing_extensions.Annotated[typing.Optional[CustomerId], FieldMetadata(alias="customerId")] = None
    customer_number: typing_extensions.Annotated[
        typing.Optional[CustomerNumberNullable], FieldMetadata(alias="customerNumber")
    ] = None
    first_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="firstName")] = pydantic.Field(
        default=None
    )
    """
    Customer/Payor first name.
    """

    identifier_fields: typing_extensions.Annotated[
        typing.Optional[Identifierfields], FieldMetadata(alias="identifierFields")
    ] = None
    last_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="lastName")] = pydantic.Field(
        default=None
    )
    """
    Customer/Payor last name.
    """

    shipping_address_1: typing_extensions.Annotated[
        typing.Optional[Shippingaddress], FieldMetadata(alias="shippingAddress1")
    ] = None
    shipping_address_2: typing_extensions.Annotated[
        typing.Optional[Shippingaddressadditional], FieldMetadata(alias="shippingAddress2")
    ] = None
    shipping_city: typing_extensions.Annotated[typing.Optional[Shippingcity], FieldMetadata(alias="shippingCity")] = (
        None
    )
    shipping_country: typing_extensions.Annotated[
        typing.Optional[Shippingcountry], FieldMetadata(alias="shippingCountry")
    ] = None
    shipping_state: typing_extensions.Annotated[
        typing.Optional[Shippingstate], FieldMetadata(alias="shippingState")
    ] = None
    shipping_zip: typing_extensions.Annotated[typing.Optional[Shippingzip], FieldMetadata(alias="shippingZip")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .achaccount import Achaccount
from .achrouting import Achrouting
from .billing_address_nullable import BillingAddressNullable
from .billing_city_nullable import BillingCityNullable
from .billing_country_nullable import BillingCountryNullable
from .billing_state_nullable import BillingStateNullable
from .billing_zip import BillingZip


class Instrument(UniversalBaseModel):
    ach_account: typing_extensions.Annotated[Achaccount, FieldMetadata(alias="achAccount")]
    ach_routing: typing_extensions.Annotated[Achrouting, FieldMetadata(alias="achRouting")]
    billing_address: typing_extensions.Annotated[
        typing.Optional[BillingAddressNullable], FieldMetadata(alias="billingAddress")
    ] = None
    billing_city: typing_extensions.Annotated[
        typing.Optional[BillingCityNullable], FieldMetadata(alias="billingCity")
    ] = None
    billing_country: typing_extensions.Annotated[
        typing.Optional[BillingCountryNullable], FieldMetadata(alias="billingCountry")
    ] = None
    billing_state: typing_extensions.Annotated[
        typing.Optional[BillingStateNullable], FieldMetadata(alias="billingState")
    ] = None
    billing_zip: typing_extensions.Annotated[typing.Optional[BillingZip], FieldMetadata(alias="billingZip")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.annualrevenue import Annualrevenue
from ..types.application_details_record import ApplicationDetailsRecord
from ..types.application_query_record import ApplicationQueryRecord
from ..types.attachments import Attachments
from ..types.avgmonthly import Avgmonthly
from ..types.baddress_1 import Baddress1
from ..types.baddress_2 import Baddress2
from ..types.bank import Bank
from ..types.bcity import Bcity
from ..types.bcountry import Bcountry
from ..types.binperson import Binperson
from ..types.binphone import Binphone
from ..types.binweb import Binweb
from ..types.boarding_business_fax import BoardingBusinessFax
from ..types.boarding_business_phone import BoardingBusinessPhone
from ..types.boarding_link_query_record import BoardingLinkQueryRecord
from ..types.bstate import Bstate
from ..types.bsummary import Bsummary
from ..types.busstartdate import Busstartdate
from ..types.bzip import Bzip
from ..types.contacts_field import ContactsField
from ..types.dbaname import Dbaname
from ..types.ein import Ein
from ..types.email import Email
from ..types.export_format import ExportFormat
from ..types.external_paypoint_id import ExternalPaypointId
from ..types.highticketamt import Highticketamt
from ..types.legalname import Legalname
from ..types.license import License
from ..types.licensestate import Licensestate
from ..types.maddress import Maddress
from ..types.maddress_1 import Maddress1
from ..types.mcc import Mcc
from ..types.mcity import Mcity
from ..types.mcountry import Mcountry
from ..types.mstate import Mstate
from ..types.mzip import Mzip
from ..types.orgid import Orgid
from ..types.own_type import OwnType
from ..types.ownership import Ownership
from ..types.payabli_api_response_00 import PayabliApiResponse00
from ..types.payabli_api_response_00_responsedatanonobject import PayabliApiResponse00Responsedatanonobject
from ..types.payout_average_monthly_volume import PayoutAverageMonthlyVolume
from ..types.payout_average_ticket_limit import PayoutAverageTicketLimit
from ..types.payout_credit_limit import PayoutCreditLimit
from ..types.payout_high_ticket_amount import PayoutHighTicketAmount
from ..types.query_boarding_apps_list_response import QueryBoardingAppsListResponse
from ..types.query_boarding_links_response import QueryBoardingLinksResponse
from ..types.recipient_email_notification import RecipientEmailNotification
from ..types.rep_code import RepCode
from ..types.rep_name import RepName
from ..types.rep_office import RepOffice
from ..types.resumable import Resumable
from ..types.services import Services
from ..types.signer_data_request import SignerDataRequest
from ..types.taxfillname import Taxfillname
from ..types.template_id import TemplateId
from ..types.ticketamt import Ticketamt
from ..types.website import Website
from ..types.whencharged import Whencharged
from ..types.whendelivered import Whendelivered
from ..types.whenprovided import Whenprovided
from ..types.whenrefunded import Whenrefunded
from .raw_client import AsyncRawBoardingClient, RawBoardingClient
from .types.add_application_request import AddApplicationRequest

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class BoardingClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawBoardingClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawBoardingClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawBoardingClient
        """
        return self._raw_client

    def add_application(
        self, *, request: AddApplicationRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Creates a boarding application in an organization. This endpoint requires an application API token.

        Parameters
        ----------
        request : AddApplicationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        from payabli import (
            ApplicationDataPayIn,
            ApplicationDataPayInBankData,
            ApplicationDataPayInContactsItem,
            ApplicationDataPayInOwnershipItem,
            ApplicationDataPayInServices,
            ApplicationDataPayInServicesAch,
            ApplicationDataPayInServicesCard,
            SignerDataRequest,
            payabli,
        )

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.boarding.add_application(
            request=ApplicationDataPayIn(
                services=ApplicationDataPayInServices(
                    ach=ApplicationDataPayInServicesAch(),
                    card=ApplicationDataPayInServicesCard(
                        accept_amex=True,
                        accept_discover=True,
                        accept_mastercard=True,
                        accept_visa=True,
                    ),
                ),
                annual_revenue=1000.0,
                average_bill_size="500",
                average_monthly_bill="5650",
                avgmonthly=1000.0,
                baddress="123 Walnut Street",
                baddress_1="Suite 103",
                bank_data=ApplicationDataPayInBankData(),
                bcity="New Vegas",
                bcountry="US",
                binperson=60,
                binphone=20,
                binweb=20,
                bstate="FL",
                bsummary="Brick and mortar store that sells office supplies",
                btype="Limited Liability Company",
                bzip="33000",
                contacts=[
                    ApplicationDataPayInContactsItem(
                        contact_email="herman@hermanscoatings.com",
                        contact_name="Herman Martinez",
                        contact_phone="3055550000",
                        contact_title="Owner",
                    )
                ],
                credit_limit="creditLimit",
                dba_name="Sunshine Gutters",
                ein="123456789",
                faxnumber="1234567890",
                highticketamt=1000.0,
                legal_name="Sunshine Services, LLC",
                license="2222222FFG",
                licstate="CA",
                maddress="123 Walnut Street",
                maddress_1="STE 900",
                mcc="7777",
                mcity="Johnson City",
                mcountry="US",
                mstate="TN",
                mzip="37615",
                org_id=123,
                ownership=[
                    ApplicationDataPayInOwnershipItem(
                        oaddress="33 North St",
                        ocity="Any City",
                        ocountry="US",
                        odriverstate="CA",
                        ostate="CA",
                        ownerdob="01/01/1990",
                        ownerdriver="CA6677778",
                        owneremail="test@email.com",
                        ownername="John Smith",
                        ownerpercent=100,
                        ownerphone_1="555888111",
                        ownerphone_2="555888111",
                        ownerssn="123456789",
                        ownertitle="CEO",
                        ozip="55555",
                    )
                ],
                phonenumber="1234567890",
                processing_region="US",
                recipient_email="josephray@example.com",
                recipient_email_notification=True,
                resumable=True,
                signer=SignerDataRequest(
                    address="33 North St",
                    address_1="STE 900",
                    city="Bristol",
                    country="US",
                    dob="01/01/1976",
                    email="test@email.com",
                    name="John Smith",
                    phone="555888111",
                    ssn="123456789",
                    state="TN",
                    zip="55555",
                    pci_attestation=True,
                    signed_document_reference="https://example.com/signed-document.pdf",
                    attestation_date="04/20/2025",
                    sign_date="04/20/2025",
                    additional_data='{"deviceId":"499585-389fj484-3jcj8hj3","session":"fifji4-fiu443-fn4843","timeWithCompany":"6 Years"}',
                ),
                startdate="01/01/1990",
                tax_fill_name="Sunshine LLC",
                template_id=22,
                ticketamt=1000.0,
                website="www.example.com",
                when_charged="When Service Provided",
                when_delivered="Over 30 Days",
                when_provided="30 Days or Less",
                when_refunded="30 Days or Less",
            ),
        )
        """
        _response = self._raw_client.add_application(request=request, request_options=request_options)
        return _response.data

    def delete_application(
        self, app_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Deletes a boarding application by ID.

        Parameters
        ----------
        app_id : int
            Boarding application ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.boarding.delete_application(
            app_id=352,
        )
        """
        _response = self._raw_client.delete_application(app_id, request_options=request_options)
        return _response.data

    def get_application(
        self, app_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ApplicationDetailsRecord:
        """
        Retrieves the details for a boarding application by ID.

        Parameters
        ----------
        app_id : int
            Boarding application ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ApplicationDetailsRecord
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.boarding.get_application(
            app_id=352,
        )
        """
        _response = self._raw_client.get_application(app_id, request_options=request_options)
        return _response.data

    def get_application_by_auth(
        self,
        x_id: str,
        *,
        email: typing.Optional[Email] = OMIT,
        reference_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ApplicationQueryRecord:
        """
        Gets a boarding application by authentication information. This endpoint requires an `application` API token.

        Parameters
        ----------
        x_id : str
            The application ID in Hex format. Find this at the end of the boarding link URL returned in a call to api/Boarding/applink/{appId}/{mail2}. For example in:  `https://boarding-sandbox.payabli.com/boarding/externalapp/load/17E`, the xId is `17E`.

        email : typing.Optional[Email]
            The email address the applicant used to save the application.

        reference_id : typing.Optional[str]
            The referenceId is sent to the applicant via email when they save the application.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ApplicationQueryRecord
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.boarding.get_application_by_auth(
            x_id="17E",
            email="admin@email.com",
            reference_id="n6UCd1f1ygG7",
        )
        """
        _response = self._raw_client.get_application_by_auth(
            x_id, email=email, reference_id=reference_id, request_options=request_options
        )
        return _response.data

    def get_by_id_link_application(
        self, boarding_link_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BoardingLinkQueryRecord:
        """
        Retrieves details for a boarding link, by ID.

        Parameters
        ----------
        boarding_link_id : int
            The boarding link ID. You can find this at the end of the boarding link reference name. For example `https://boarding.payabli.com/boarding/app/myorgaccountname-00091`. The ID is `91`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BoardingLinkQueryRecord
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.boarding.get_by_id_link_application(
            boarding_link_id=91,
        )
        """
        _response = self._raw_client.get_by_id_link_application(boarding_link_id, request_options=request_options)
        return _response.data

    def get_by_template_id_link_application(
        self, template_id: float, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BoardingLinkQueryRecord:
        """
        Get details for a boarding link using the boarding template ID. This endpoint requires an application API token.

        Parameters
        ----------
        template_id : float
            The boarding template ID. You can find this at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BoardingLinkQueryRecord
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.boarding.get_by_template_id_link_application(
            template_id=80.0,
        )
        """
        _response = self._raw_client.get_by_template_id_link_application(template_id, request_options=request_options)
        return _response.data

    def get_external_application(
        self,
        app_id: int,
        mail_2: str,
        *,
        send_email: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponse00:
        """
        Retrieves a link and the verification code used to log into an existing boarding application. You can also use this endpoint to send a link and referenceId for an existing boarding application to an email address. The recipient can use the referenceId and email address to access and edit the application.

        Parameters
        ----------
        app_id : int
            Boarding application ID.

        mail_2 : str
            Email address used to access the application. If `sendEmail` parameter is true, a link to the application is sent to this email address.

        send_email : typing.Optional[bool]
            If `true`, sends an email that includes the link to the application to the `mail2` address. Defaults to `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.boarding.get_external_application(
            app_id=352,
            mail_2="mail2",
        )
        """
        _response = self._raw_client.get_external_application(
            app_id, mail_2, send_email=send_email, request_options=request_options
        )
        return _response.data

    def get_link_application(
        self, boarding_link_reference: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BoardingLinkQueryRecord:
        """
        Retrieves the details for a boarding link, by reference name. This endpoint requires an application API token.

        Parameters
        ----------
        boarding_link_reference : str
            The boarding link reference name. You can find this at the end of the boarding link URL. For example `https://boarding.payabli.com/boarding/app/myorgaccountname-00091`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BoardingLinkQueryRecord
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.boarding.get_link_application(
            boarding_link_reference="myorgaccountname-00091",
        )
        """
        _response = self._raw_client.get_link_application(boarding_link_reference, request_options=request_options)
        return _response.data

    def list_applications(
        self,
        org_id: int,
        *,
        export_format: typing.Optional[ExportFormat] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryBoardingAppsListResponse:
        """
        Returns a list of boarding applications for an organization. Use filters to limit results. Include the `exportFormat` query parameter to return the results as a file instead of a JSON response.

        Parameters
        ----------
        org_id : int
            The numeric identifier for organization, assigned by Payabli.

        export_format : typing.Optional[ExportFormat]

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Collection of field names, conditions, and values used to filter the query

            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:
            - `createdAt` (gt, ge, lt, le, eq, ne)
            - `startDate` (gt, ge, lt, le, eq, ne)
            - `dbaname` (ct, nct)
            - `legalname` (ct, nct)
            - `ein` (ct, nct)
            - `address` (ct, nct)
            - `city` (ct, nct)
            - `state` (ct, nct)
            - `phone` (ct, nct)
            - `mcc` (ct, nct)
            - `owntype` (ct, nct)
            - `ownerName` (ct, nct)
            - `contactName` (ct, nct)
            - `status` (in, nin, eq,ne)
            - `orgParentname` (ct, nct)
            - `externalpaypointID` (ct, nct, eq, ne)
            - `repCode` (ct, nct, eq, ne)
            - `repName` (ct, nct, eq, ne)
            - `repOffice` (ct, nct, eq, ne)
            List of comparison accepted - enclosed between parentheses:
            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array
            - nin => not inside array

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QueryBoardingAppsListResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.boarding.list_applications(
            org_id=123,
            from_record=251,
            limit_record=0,
            sort_by="desc(field_name)",
        )
        """
        _response = self._raw_client.list_applications(
            org_id,
            export_format=export_format,
            from_record=from_record,
            limit_record=limit_record,
            parameters=parameters,
            sort_by=sort_by,
            request_options=request_options,
        )
        return _response.data

    def list_boarding_links(
        self,
        org_id: int,
        *,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryBoardingLinksResponse:
        """
        Return a list of boarding links for an organization. Use filters to limit results.

        Parameters
        ----------
        org_id : int
            The numeric identifier for organization, assigned by Payabli.

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Collection of field names, conditions, and values used to filter the query

            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:
            - `lastUpdated` (gt, ge, lt, le, eq, ne)
            - `templateName` (ct, nct)
            - `referenceName` (ct, nct)
            - `acceptRegister` (eq, ne)
            - `acceptAuth` (eq, ne)
            - `templateCode` (ct, nct)
            - `templateId` (eq, ne)
            - `orgParentname` (ct, nct)

            List of comparison accepted - enclosed between parentheses:
            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array
            - nin => not inside array

            List of parameters accepted:
            - limitRecord : max number of records for query (default="20", "0" or negative value for all)
            - fromRecord : initial record in query

            Example: templateName(ct)=hoa return all records with template title containing "hoa"

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QueryBoardingLinksResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.boarding.list_boarding_links(
            org_id=123,
            from_record=251,
            limit_record=0,
            sort_by="desc(field_name)",
        )
        """
        _response = self._raw_client.list_boarding_links(
            org_id,
            from_record=from_record,
            limit_record=limit_record,
            parameters=parameters,
            sort_by=sort_by,
            request_options=request_options,
        )
        return _response.data

    def update_application(
        self,
        app_id: int,
        *,
        services: typing.Optional[Services] = OMIT,
        annual_revenue: typing.Optional[Annualrevenue] = OMIT,
        attachments: typing.Optional[Attachments] = OMIT,
        avgmonthly: typing.Optional[Avgmonthly] = OMIT,
        baddress: typing.Optional[Baddress1] = OMIT,
        baddress_1: typing.Optional[Baddress2] = OMIT,
        bank_data: typing.Optional[Bank] = OMIT,
        bcity: typing.Optional[Bcity] = OMIT,
        bcountry: typing.Optional[Bcountry] = OMIT,
        binperson: typing.Optional[Binperson] = OMIT,
        binphone: typing.Optional[Binphone] = OMIT,
        binweb: typing.Optional[Binweb] = OMIT,
        bstate: typing.Optional[Bstate] = OMIT,
        bsummary: typing.Optional[Bsummary] = OMIT,
        btype: typing.Optional[OwnType] = OMIT,
        bzip: typing.Optional[Bzip] = OMIT,
        contacts: typing.Optional[ContactsField] = OMIT,
        dbaname: typing.Optional[Dbaname] = OMIT,
        ein: typing.Optional[Ein] = OMIT,
        external_paypoint_id: typing.Optional[ExternalPaypointId] = OMIT,
        faxnumber: typing.Optional[BoardingBusinessFax] = OMIT,
        highticketamt: typing.Optional[Highticketamt] = OMIT,
        legalname: typing.Optional[Legalname] = OMIT,
        license: typing.Optional[License] = OMIT,
        licstate: typing.Optional[Licensestate] = OMIT,
        maddress: typing.Optional[Maddress] = OMIT,
        maddress_1: typing.Optional[Maddress1] = OMIT,
        mcc: typing.Optional[Mcc] = OMIT,
        mcity: typing.Optional[Mcity] = OMIT,
        mcountry: typing.Optional[Mcountry] = OMIT,
        mstate: typing.Optional[Mstate] = OMIT,
        mzip: typing.Optional[Mzip] = OMIT,
        org_id: typing.Optional[Orgid] = OMIT,
        ownership: typing.Optional[Ownership] = OMIT,
        payout_average_monthly_volume: typing.Optional[PayoutAverageMonthlyVolume] = OMIT,
        payout_average_ticket_limit: typing.Optional[PayoutAverageTicketLimit] = OMIT,
        payout_credit_limit: typing.Optional[PayoutCreditLimit] = OMIT,
        payout_high_ticket_amount: typing.Optional[PayoutHighTicketAmount] = OMIT,
        phonenumber: typing.Optional[BoardingBusinessPhone] = OMIT,
        recipient_email: typing.Optional[Email] = OMIT,
        recipient_email_notification: typing.Optional[RecipientEmailNotification] = OMIT,
        resumable: typing.Optional[Resumable] = OMIT,
        signer: typing.Optional[SignerDataRequest] = OMIT,
        startdate: typing.Optional[Busstartdate] = OMIT,
        taxfillname: typing.Optional[Taxfillname] = OMIT,
        template_id: typing.Optional[TemplateId] = OMIT,
        ticketamt: typing.Optional[Ticketamt] = OMIT,
        website: typing.Optional[Website] = OMIT,
        when_charged: typing.Optional[Whencharged] = OMIT,
        when_delivered: typing.Optional[Whendelivered] = OMIT,
        when_provided: typing.Optional[Whenprovided] = OMIT,
        when_refunded: typing.Optional[Whenrefunded] = OMIT,
        rep_code: typing.Optional[RepCode] = OMIT,
        rep_name: typing.Optional[RepName] = OMIT,
        rep_office: typing.Optional[RepOffice] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Updates a boarding application by ID. This endpoint requires an application API token.

        Parameters
        ----------
        app_id : int
            Boarding application ID.

        services : typing.Optional[Services]

        annual_revenue : typing.Optional[Annualrevenue]

        attachments : typing.Optional[Attachments]

        avgmonthly : typing.Optional[Avgmonthly]

        baddress : typing.Optional[Baddress1]

        baddress_1 : typing.Optional[Baddress2]

        bank_data : typing.Optional[Bank]

        bcity : typing.Optional[Bcity]

        bcountry : typing.Optional[Bcountry]

        binperson : typing.Optional[Binperson]

        binphone : typing.Optional[Binphone]

        binweb : typing.Optional[Binweb]

        bstate : typing.Optional[Bstate]

        bsummary : typing.Optional[Bsummary]

        btype : typing.Optional[OwnType]

        bzip : typing.Optional[Bzip]

        contacts : typing.Optional[ContactsField]

        dbaname : typing.Optional[Dbaname]

        ein : typing.Optional[Ein]

        external_paypoint_id : typing.Optional[ExternalPaypointId]

        faxnumber : typing.Optional[BoardingBusinessFax]

        highticketamt : typing.Optional[Highticketamt]

        legalname : typing.Optional[Legalname]

        license : typing.Optional[License]

        licstate : typing.Optional[Licensestate]

        maddress : typing.Optional[Maddress]

        maddress_1 : typing.Optional[Maddress1]

        mcc : typing.Optional[Mcc]

        mcity : typing.Optional[Mcity]

        mcountry : typing.Optional[Mcountry]

        mstate : typing.Optional[Mstate]

        mzip : typing.Optional[Mzip]

        org_id : typing.Optional[Orgid]

        ownership : typing.Optional[Ownership]

        payout_average_monthly_volume : typing.Optional[PayoutAverageMonthlyVolume]

        payout_average_ticket_limit : typing.Optional[PayoutAverageTicketLimit]

        payout_credit_limit : typing.Optional[PayoutCreditLimit]

        payout_high_ticket_amount : typing.Optional[PayoutHighTicketAmount]

        phonenumber : typing.Optional[BoardingBusinessPhone]

        recipient_email : typing.Optional[Email]
            Email address for the applicant. This is used to send the applicant a boarding link.

        recipient_email_notification : typing.Optional[RecipientEmailNotification]

        resumable : typing.Optional[Resumable]

        signer : typing.Optional[SignerDataRequest]

        startdate : typing.Optional[Busstartdate]

        taxfillname : typing.Optional[Taxfillname]

        template_id : typing.Optional[TemplateId]

        ticketamt : typing.Optional[Ticketamt]

        website : typing.Optional[Website]

        when_charged : typing.Optional[Whencharged]

        when_delivered : typing.Optional[Whendelivered]

        when_provided : typing.Optional[Whenprovided]

        when_refunded : typing.Optional[Whenrefunded]

        rep_code : typing.Optional[RepCode]

        rep_name : typing.Optional[RepName]

        rep_office : typing.Optional[RepOffice]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.boarding.update_application(
            app_id=352,
        )
        """
        _response = self._raw_client.update_application(
            app_id,
            services=services,
            annual_revenue=annual_revenue,
            attachments=attachments,
            avgmonthly=avgmonthly,
            baddress=baddress,
            baddress_1=baddress_1,
            bank_data=bank_data,
            bcity=bcity,
            bcountry=bcountry,
            binperson=binperson,
            binphone=binphone,
            binweb=binweb,
            bstate=bstate,
            bsummary=bsummary,
            btype=btype,
            bzip=bzip,
            contacts=contacts,
            dbaname=dbaname,
            ein=ein,
            external_paypoint_id=external_paypoint_id,
            faxnumber=faxnumber,
            highticketamt=highticketamt,
            legalname=legalname,
            license=license,
            licstate=licstate,
            maddress=maddress,
            maddress_1=maddress_1,
            mcc=mcc,
            mcity=mcity,
            mcountry=mcountry,
            mstate=mstate,
            mzip=mzip,
            org_id=org_id,
            ownership=ownership,
            payout_average_monthly_volume=payout_average_monthly_volume,
            payout_average_ticket_limit=payout_average_ticket_limit,
            payout_credit_limit=payout_credit_limit,
            payout_high_ticket_amount=payout_high_ticket_amount,
            phonenumber=phonenumber,
            recipient_email=recipient_email,
            recipient_email_notification=recipient_email_notification,
            resumable=resumable,
            signer=signer,
            startdate=startdate,
            taxfillname=taxfillname,
            template_id=template_id,
            ticketamt=ticketamt,
            website=website,
            when_charged=when_charged,
            when_delivered=when_delivered,
            when_provided=when_provided,
            when_refunded=when_refunded,
            rep_code=rep_code,
            rep_name=rep_name,
            rep_office=rep_office,
            request_options=request_options,
        )
        return _response.data


class AsyncBoardingClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawBoardingClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawBoardingClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawBoardingClient
        """
        return self._raw_client

    async def add_application(
        self, *, request: AddApplicationRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Creates a boarding application in an organization. This endpoint requires an application API token.

        Parameters
        ----------
        request : AddApplicationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        import asyncio

        from payabli import (
            ApplicationDataPayIn,
            ApplicationDataPayInBankData,
            ApplicationDataPayInContactsItem,
            ApplicationDataPayInOwnershipItem,
            ApplicationDataPayInServices,
            ApplicationDataPayInServicesAch,
            ApplicationDataPayInServicesCard,
            Asyncpayabli,
            SignerDataRequest,
        )

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.boarding.add_application(
                request=ApplicationDataPayIn(
                    services=ApplicationDataPayInServices(
                        ach=ApplicationDataPayInServicesAch(),
                        card=ApplicationDataPayInServicesCard(
                            accept_amex=True,
                            accept_discover=True,
                            accept_mastercard=True,
                            accept_visa=True,
                        ),
                    ),
                    annual_revenue=1000.0,
                    average_bill_size="500",
                    average_monthly_bill="5650",
                    avgmonthly=1000.0,
                    baddress="123 Walnut Street",
                    baddress_1="Suite 103",
                    bank_data=ApplicationDataPayInBankData(),
                    bcity="New Vegas",
                    bcountry="US",
                    binperson=60,
                    binphone=20,
                    binweb=20,
                    bstate="FL",
                    bsummary="Brick and mortar store that sells office supplies",
                    btype="Limited Liability Company",
                    bzip="33000",
                    contacts=[
                        ApplicationDataPayInContactsItem(
                            contact_email="herman@hermanscoatings.com",
                            contact_name="Herman Martinez",
                            contact_phone="3055550000",
                            contact_title="Owner",
                        )
                    ],
                    credit_limit="creditLimit",
                    dba_name="Sunshine Gutters",
                    ein="123456789",
                    faxnumber="1234567890",
                    highticketamt=1000.0,
                    legal_name="Sunshine Services, LLC",
                    license="2222222FFG",
                    licstate="CA",
                    maddress="123 Walnut Street",
                    maddress_1="STE 900",
                    mcc="7777",
                    mcity="Johnson City",
                    mcountry="US",
                    mstate="TN",
                    mzip="37615",
                    org_id=123,
                    ownership=[
                        ApplicationDataPayInOwnershipItem(
                            oaddress="33 North St",
                            ocity="Any City",
                            ocountry="US",
                            odriverstate="CA",
                            ostate="CA",
                            ownerdob="01/01/1990",
                            ownerdriver="CA6677778",
                            owneremail="test@email.com",
                            ownername="John Smith",
                            ownerpercent=100,
                            ownerphone_1="555888111",
                            ownerphone_2="555888111",
                            ownerssn="123456789",
                            ownertitle="CEO",
                            ozip="55555",
                        )
                    ],
                    phonenumber="1234567890",
                    processing_region="US",
                    recipient_email="josephray@example.com",
                    recipient_email_notification=True,
                    resumable=True,
                    signer=SignerDataRequest(
                        address="33 North St",
                        address_1="STE 900",
                        city="Bristol",
                        country="US",
                        dob="01/01/1976",
                        email="test@email.com",
                        name="John Smith",
                        phone="555888111",
                        ssn="123456789",
                        state="TN",
                        zip="55555",
                        pci_attestation=True,
                        signed_document_reference="https://example.com/signed-document.pdf",
                        attestation_date="04/20/2025",
                        sign_date="04/20/2025",
                        additional_data='{"deviceId":"499585-389fj484-3jcj8hj3","session":"fifji4-fiu443-fn4843","timeWithCompany":"6 Years"}',
                    ),
                    startdate="01/01/1990",
                    tax_fill_name="Sunshine LLC",
                    template_id=22,
                    ticketamt=1000.0,
                    website="www.example.com",
                    when_charged="When Service Provided",
                    when_delivered="Over 30 Days",
                    when_provided="30 Days or Less",
                    when_refunded="30 Days or Less",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.add_application(request=request, request_options=request_options)
        return _response.data

    async def delete_application(
        self, app_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Deletes a boarding application by ID.

        Parameters
        ----------
        app_id : int
            Boarding application ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.boarding.delete_application(
                app_id=352,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_application(app_id, request_options=request_options)
        return _response.data

    async def get_application(
        self, app_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ApplicationDetailsRecord:
        """
        Retrieves the details for a boarding application by ID.

        Parameters
        ----------
        app_id : int
            Boarding application ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ApplicationDetailsRecord
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.boarding.get_application(
                app_id=352,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_application(app_id, request_options=request_options)
        return _response.data

    async def get_application_by_auth(
        self,
        x_id: str,
        *,
        email: typing.Optional[Email] = OMIT,
        reference_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ApplicationQueryRecord:
        """
        Gets a boarding application by authentication information. This endpoint requires an `application` API token.

        Parameters
        ----------
        x_id : str
            The application ID in Hex format. Find this at the end of the boarding link URL returned in a call to api/Boarding/applink/{appId}/{mail2}. For example in:  `https://boarding-sandbox.payabli.com/boarding/externalapp/load/17E`, the xId is `17E`.

        email : typing.Optional[Email]
            The email address the applicant used to save the application.

        reference_id : typing.Optional[str]
            The referenceId is sent to the applicant via email when they save the application.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ApplicationQueryRecord
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.boarding.get_application_by_auth(
                x_id="17E",
                email="admin@email.com",
                reference_id="n6UCd1f1ygG7",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_application_by_auth(
            x_id, email=email, reference_id=reference_id, request_options=request_options
        )
        return _response.data

    async def get_by_id_link_application(
        self, boarding_link_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BoardingLinkQueryRecord:
        """
        Retrieves details for a boarding link, by ID.

        Parameters
        ----------
        boarding_link_id : int
            The boarding link ID. You can find this at the end of the boarding link reference name. For example `https://boarding.payabli.com/boarding/app/myorgaccountname-00091`. The ID is `91`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BoardingLinkQueryRecord
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.boarding.get_by_id_link_application(
                boarding_link_id=91,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_by_id_link_application(boarding_link_id, request_options=request_options)
        return _response.data

    async def get_by_template_id_link_application(
        self, template_id: float, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BoardingLinkQueryRecord:
        """
        Get details for a boarding link using the boarding template ID. This endpoint requires an application API token.

        Parameters
        ----------
        template_id : float
            The boarding template ID. You can find this at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BoardingLinkQueryRecord
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.boarding.get_by_template_id_link_application(
                template_id=80.0,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_by_template_id_link_application(
            template_id, request_options=request_options
        )
        return _response.data

    async def get_external_application(
        self,
        app_id: int,
        mail_2: str,
        *,
        send_email: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponse00:
        """
        Retrieves a link and the verification code used to log into an existing boarding application. You can also use this endpoint to send a link and referenceId for an existing boarding application to an email address. The recipient can use the referenceId and email address to access and edit the application.

        Parameters
        ----------
        app_id : int
            Boarding application ID.

        mail_2 : str
            Email address used to access the application. If `sendEmail` parameter is true, a link to the application is sent to this email address.

        send_email : typing.Optional[bool]
            If `true`, sends an email that includes the link to the application to the `mail2` address. Defaults to `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.boarding.get_external_application(
                app_id=352,
                mail_2="mail2",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_external_application(
            app_id, mail_2, send_email=send_email, request_options=request_options
        )
        return _response.data

    async def get_link_application(
        self, boarding_link_reference: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BoardingLinkQueryRecord:
        """
        Retrieves the details for a boarding link, by reference name. This endpoint requires an application API token.

        Parameters
        ----------
        boarding_link_reference : str
            The boarding link reference name. You can find this at the end of the boarding link URL. For example `https://boarding.payabli.com/boarding/app/myorgaccountname-00091`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BoardingLinkQueryRecord
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.boarding.get_link_application(
                boarding_link_reference="myorgaccountname-00091",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_link_application(
            boarding_link_reference, request_options=request_options
        )
        return _response.data

    async def list_applications(
        self,
        org_id: int,
        *,
        export_format: typing.Optional[ExportFormat] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryBoardingAppsListResponse:
        """
        Returns a list of boarding applications for an organization. Use filters to limit results. Include the `exportFormat` query parameter to return the results as a file instead of a JSON response.

        Parameters
        ----------
        org_id : int
            The numeric identifier for organization, assigned by Payabli.

        export_format : typing.Optional[ExportFormat]

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Collection of field names, conditions, and values used to filter the query

            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:
            - `createdAt` (gt, ge, lt, le, eq, ne)
            - `startDate` (gt, ge, lt, le, eq, ne)
            - `dbaname` (ct, nct)
            - `legalname` (ct, nct)
            - `ein` (ct, nct)
            - `address` (ct, nct)
            - `city` (ct, nct)
            - `state` (ct, nct)
            - `phone` (ct, nct)
            - `mcc` (ct, nct)
            - `owntype` (ct, nct)
            - `ownerName` (ct, nct)
            - `contactName` (ct, nct)
            - `status` (in, nin, eq,ne)
            - `orgParentname` (ct, nct)
            - `externalpaypointID` (ct, nct, eq, ne)
            - `repCode` (ct, nct, eq, ne)
            - `repName` (ct, nct, eq, ne)
            - `repOffice` (ct, nct, eq, ne)
            List of comparison accepted - enclosed between parentheses:
            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array
            - nin => not inside array

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QueryBoardingAppsListResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.boarding.list_applications(
                org_id=123,
                from_record=251,
                limit_record=0,
                sort_by="desc(field_name)",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_applications(
            org_id,
            export_format=export_format,
            from_record=from_record,
            limit_record=limit_record,
            parameters=parameters,
            sort_by=sort_by,
            request_options=request_options,
        )
        return _response.data

    async def list_boarding_links(
        self,
        org_id: int,
        *,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryBoardingLinksResponse:
        """
        Return a list of boarding links for an organization. Use filters to limit results.

        Parameters
        ----------
        org_id : int
            The numeric identifier for organization, assigned by Payabli.

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Collection of field names, conditions, and values used to filter the query

            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:
            - `lastUpdated` (gt, ge, lt, le, eq, ne)
            - `templateName` (ct, nct)
            - `referenceName` (ct, nct)
            - `acceptRegister` (eq, ne)
            - `acceptAuth` (eq, ne)
            - `templateCode` (ct, nct)
            - `templateId` (eq, ne)
            - `orgParentname` (ct, nct)

            List of comparison accepted - enclosed between parentheses:
            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array
            - nin => not inside array

            List of parameters accepted:
            - limitRecord : max number of records for query (default="20", "0" or negative value for all)
            - fromRecord : initial record in query

            Example: templateName(ct)=hoa return all records with template title containing "hoa"

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QueryBoardingLinksResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.boarding.list_boarding_links(
                org_id=123,
                from_record=251,
                limit_record=0,
                sort_by="desc(field_name)",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_boarding_links(
            org_id,
            from_record=from_record,
            limit_record=limit_record,
            parameters=parameters,
            sort_by=sort_by,
            request_options=request_options,
        )
        return _response.data

    async def update_application(
        self,
        app_id: int,
        *,
        services: typing.Optional[Services] = OMIT,
        annual_revenue: typing.Optional[Annualrevenue] = OMIT,
        attachments: typing.Optional[Attachments] = OMIT,
        avgmonthly: typing.Optional[Avgmonthly] = OMIT,
        baddress: typing.Optional[Baddress1] = OMIT,
        baddress_1: typing.Optional[Baddress2] = OMIT,
        bank_data: typing.Optional[Bank] = OMIT,
        bcity: typing.Optional[Bcity] = OMIT,
        bcountry: typing.Optional[Bcountry] = OMIT,
        binperson: typing.Optional[Binperson] = OMIT,
        binphone: typing.Optional[Binphone] = OMIT,
        binweb: typing.Optional[Binweb] = OMIT,
        bstate: typing.Optional[Bstate] = OMIT,
        bsummary: typing.Optional[Bsummary] = OMIT,
        btype: typing.Optional[OwnType] = OMIT,
        bzip: typing.Optional[Bzip] = OMIT,
        contacts: typing.Optional[ContactsField] = OMIT,
        dbaname: typing.Optional[Dbaname] = OMIT,
        ein: typing.Optional[Ein] = OMIT,
        external_paypoint_id: typing.Optional[ExternalPaypointId] = OMIT,
        faxnumber: typing.Optional[BoardingBusinessFax] = OMIT,
        highticketamt: typing.Optional[Highticketamt] = OMIT,
        legalname: typing.Optional[Legalname] = OMIT,
        license: typing.Optional[License] = OMIT,
        licstate: typing.Optional[Licensestate] = OMIT,
        maddress: typing.Optional[Maddress] = OMIT,
        maddress_1: typing.Optional[Maddress1] = OMIT,
        mcc: typing.Optional[Mcc] = OMIT,
        mcity: typing.Optional[Mcity] = OMIT,
        mcountry: typing.Optional[Mcountry] = OMIT,
        mstate: typing.Optional[Mstate] = OMIT,
        mzip: typing.Optional[Mzip] = OMIT,
        org_id: typing.Optional[Orgid] = OMIT,
        ownership: typing.Optional[Ownership] = OMIT,
        payout_average_monthly_volume: typing.Optional[PayoutAverageMonthlyVolume] = OMIT,
        payout_average_ticket_limit: typing.Optional[PayoutAverageTicketLimit] = OMIT,
        payout_credit_limit: typing.Optional[PayoutCreditLimit] = OMIT,
        payout_high_ticket_amount: typing.Optional[PayoutHighTicketAmount] = OMIT,
        phonenumber: typing.Optional[BoardingBusinessPhone] = OMIT,
        recipient_email: typing.Optional[Email] = OMIT,
        recipient_email_notification: typing.Optional[RecipientEmailNotification] = OMIT,
        resumable: typing.Optional[Resumable] = OMIT,
        signer: typing.Optional[SignerDataRequest] = OMIT,
        startdate: typing.Optional[Busstartdate] = OMIT,
        taxfillname: typing.Optional[Taxfillname] = OMIT,
        template_id: typing.Optional[TemplateId] = OMIT,
        ticketamt: typing.Optional[Ticketamt] = OMIT,
        website: typing.Optional[Website] = OMIT,
        when_charged: typing.Optional[Whencharged] = OMIT,
        when_delivered: typing.Optional[Whendelivered] = OMIT,
        when_provided: typing.Optional[Whenprovided] = OMIT,
        when_refunded: typing.Optional[Whenrefunded] = OMIT,
        rep_code: typing.Optional[RepCode] = OMIT,
        rep_name: typing.Optional[RepName] = OMIT,
        rep_office: typing.Optional[RepOffice] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponse00Responsedatanonobject:
        """
        Updates a boarding application by ID. This endpoint requires an application API token.

        Parameters
        ----------
        app_id : int
            Boarding application ID.

        services : typing.Optional[Services]

        annual_revenue : typing.Optional[Annualrevenue]

        attachments : typing.Optional[Attachments]

        avgmonthly : typing.Optional[Avgmonthly]

        baddress : typing.Optional[Baddress1]

        baddress_1 : typing.Optional[Baddress2]

        bank_data : typing.Optional[Bank]

        bcity : typing.Optional[Bcity]

        bcountry : typing.Optional[Bcountry]

        binperson : typing.Optional[Binperson]

        binphone : typing.Optional[Binphone]

        binweb : typing.Optional[Binweb]

        bstate : typing.Optional[Bstate]

        bsummary : typing.Optional[Bsummary]

        btype : typing.Optional[OwnType]

        bzip : typing.Optional[Bzip]

        contacts : typing.Optional[ContactsField]

        dbaname : typing.Optional[Dbaname]

        ein : typing.Optional[Ein]

        external_paypoint_id : typing.Optional[ExternalPaypointId]

        faxnumber : typing.Optional[BoardingBusinessFax]

        highticketamt : typing.Optional[Highticketamt]

        legalname : typing.Optional[Legalname]

        license : typing.Optional[License]

        licstate : typing.Optional[Licensestate]

        maddress : typing.Optional[Maddress]

        maddress_1 : typing.Optional[Maddress1]

        mcc : typing.Optional[Mcc]

        mcity : typing.Optional[Mcity]

        mcountry : typing.Optional[Mcountry]

        mstate : typing.Optional[Mstate]

        mzip : typing.Optional[Mzip]

        org_id : typing.Optional[Orgid]

        ownership : typing.Optional[Ownership]

        payout_average_monthly_volume : typing.Optional[PayoutAverageMonthlyVolume]

        payout_average_ticket_limit : typing.Optional[PayoutAverageTicketLimit]

        payout_credit_limit : typing.Optional[PayoutCreditLimit]

        payout_high_ticket_amount : typing.Optional[PayoutHighTicketAmount]

        phonenumber : typing.Optional[BoardingBusinessPhone]

        recipient_email : typing.Optional[Email]
            Email address for the applicant. This is used to send the applicant a boarding link.

        recipient_email_notification : typing.Optional[RecipientEmailNotification]

        resumable : typing.Optional[Resumable]

        signer : typing.Optional[SignerDataRequest]

        startdate : typing.Optional[Busstartdate]

        taxfillname : typing.Optional[Taxfillname]

        template_id : typing.Optional[TemplateId]

        ticketamt : typing.Optional[Ticketamt]

        website : typing.Optional[Website]

        when_charged : typing.Optional[Whencharged]

        when_delivered : typing.Optional[Whendelivered]

        when_provided : typing.Optional[Whenprovided]

        when_refunded : typing.Optional[Whenrefunded]

        rep_code : typing.Optional[RepCode]

        rep_name : typing.Optional[RepName]

        rep_office : typing.Optional[RepOffice]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse00Responsedatanonobject
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.boarding.update_application(
                app_id=352,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_application(
            app_id,
            services=services,
            annual_revenue=annual_revenue,
            attachments=attachments,
            avgmonthly=avgmonthly,
            baddress=baddress,
            baddress_1=baddress_1,
            bank_data=bank_data,
            bcity=bcity,
            bcountry=bcountry,
            binperson=binperson,
            binphone=binphone,
            binweb=binweb,
            bstate=bstate,
            bsummary=bsummary,
            btype=btype,
            bzip=bzip,
            contacts=contacts,
            dbaname=dbaname,
            ein=ein,
            external_paypoint_id=external_paypoint_id,
            faxnumber=faxnumber,
            highticketamt=highticketamt,
            legalname=legalname,
            license=license,
            licstate=licstate,
            maddress=maddress,
            maddress_1=maddress_1,
            mcc=mcc,
            mcity=mcity,
            mcountry=mcountry,
            mstate=mstate,
            mzip=mzip,
            org_id=org_id,
            ownership=ownership,
            payout_average_monthly_volume=payout_average_monthly_volume,
            payout_average_ticket_limit=payout_average_ticket_limit,
            payout_credit_limit=payout_credit_limit,
            payout_high_ticket_amount=payout_high_ticket_amount,
            phonenumber=phonenumber,
            recipient_email=recipient_email,
            recipient_email_notification=recipient_email_notification,
            resumable=resumable,
            signer=signer,
            startdate=startdate,
            taxfillname=taxfillname,
            template_id=template_id,
            ticketamt=ticketamt,
            website=website,
            when_charged=when_charged,
            when_delivered=when_delivered,
            when_provided=when_provided,
            when_refunded=when_refunded,
            rep_code=rep_code,
            rep_name=rep_name,
            rep_office=rep_office,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .batch_detail_response_record import BatchDetailResponseRecord
    from .batch_detail_response_summary import BatchDetailResponseSummary
    from .limit_record import LimitRecord
    from .list_organizations_response import ListOrganizationsResponse
    from .query_batches_detail_response import QueryBatchesDetailResponse
    from .query_batches_response import QueryBatchesResponse
    from .query_batches_response_records_item import QueryBatchesResponseRecordsItem
    from .query_batches_transfer import QueryBatchesTransfer
    from .query_transfer_detail_response import QueryTransferDetailResponse
    from .query_transfer_response import QueryTransferResponse
    from .query_transfer_summary import QueryTransferSummary
    from .transfer_detail_record import TransferDetailRecord
_dynamic_imports: typing.Dict[str, str] = {
    "BatchDetailResponseRecord": ".batch_detail_response_record",
    "BatchDetailResponseSummary": ".batch_detail_response_summary",
    "LimitRecord": ".limit_record",
    "ListOrganizationsResponse": ".list_organizations_response",
    "QueryBatchesDetailResponse": ".query_batches_detail_response",
    "QueryBatchesResponse": ".query_batches_response",
    "QueryBatchesResponseRecordsItem": ".query_batches_response_records_item",
    "QueryBatchesTransfer": ".query_batches_transfer",
    "QueryTransferDetailResponse": ".query_transfer_detail_response",
    "QueryTransferResponse": ".query_transfer_response",
    "QueryTransferSummary": ".query_transfer_summary",
    "TransferDetailRecord": ".transfer_detail_record",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "BatchDetailResponseRecord",
    "BatchDetailResponseSummary",
    "LimitRecord",
    "ListOrganizationsResponse",
    "QueryBatchesDetailResponse",
    "QueryBatchesResponse",
    "QueryBatchesResponseRecordsItem",
    "QueryBatchesTransfer",
    "QueryTransferDetailResponse",
    "QueryTransferResponse",
    "QueryTransferSummary",
    "TransferDetailRecord",
]
